<?php
/**
 * Philia Core API
 * This class handles API requests to the configured Philia API endpoint.
 * 
 * Package: Philia
 * Author: Philia
 * Last Modified Time: 2024/12/02 15:43:09
 * License: MIT
 */

namespace Philia_API;

class Philia_Core_API {
    /**
     * The API endpoint URL retrieved from the settings.
     * 
     * @var string
     */
    protected static $endpoint_url;

    /**
     * The base path for the API.
     * 
     * @var string
     */
    protected static $api_base_path = '/v1/api/';

    /**
     * Static instance of the class for the singleton pattern.
     * 
     * @var Philia_Core_API|null
     */
    protected static $_instance = null;

    /**
     * Singleton instance
     * 
     * Ensures only one instance of the class is created.
     * 
     * @return Philia_Core_API The instance of this class.
     */
    public static function instance() {
        if (is_null(self::$_instance)) {
            self::$_instance = new self();
        }
        return self::$_instance;
    }

    /**
     * Constructor.
     * Initializes the class by retrieving the API URL from WordPress settings.
     * 
     * @throws \Exception If the API URL is not configured in the settings.
     */
    private function __construct() 
    {
        // Get API settings from WordPress options
        $settings = get_option('philia_api_settings');
        if( !empty( $settings ) && !empty( $settings['api_url'] ) )
            self::$endpoint_url = rtrim( esc_url( $settings['api_url'] ) ?? '', '');  // Set base URL without concatenation

        // Throw an error if the URL is missing
        if (!empty( $settings ) && empty($settings['api_url'])) {
            Philia_API_Log::error("API URL is not configured in the settings.");
            // throw new \Exception(__('API URL is not configured. Please set it in the Philia API settings.', 'philia'));
        }
    }

    /**
     * Prevent cloning of the singleton instance.
     */
    private function __clone()
    {
        // Prevent clone of the instance
    }

    /**
     * Get the API endpoint URL.
     * 
     * @return string The configured API endpoint URL.
     */
    public static function get_endpoint()
    {
        return self::$endpoint_url . self::$api_base_path; // Concatenate the base path when retrieving the endpoint
    }

    /**
     * Adds an item to the specified option list if it doesn't already exist.
     *
     * @param string $option_name The name of the option to modify.
     * @param mixed $item The item to add to the option list.
     * @return bool True if added, false if already exists.
     */
    public static function add_item($option_name, $item)
    {
        $items = get_option($option_name, []);
        if( !empty( $items ) )
        {
            if (!in_array($item, $items)) {
                $items[] = $item;
                update_option($option_name, $items);
                return true; // Item added
            }
        }
        else
        {
            $items = [ $item ];
            update_option($option_name, $items);
            return true; // Item added
        }
        
        return false; // Item already exists
    }

    /**
     * Removes an item from the specified option list if it exists.
     *
     * @param string $option_name The name of the option to modify.
     * @param mixed $item The item to remove from the option list.
     * @return bool True if removed, false if not found.
     */
    public static function remove_item($option_name, $item)
    {
        $items = get_option($option_name, []);
        if ( !empty( $items ) && in_array($item, $items)) {
            $updated_items = array_diff($items, [$item]);
            update_option($option_name, $updated_items);
            return true; // Item removed
        }
        return false; // Item not found
    }

    /**
     * Send a POST, PUT, or DELETE request to the API.
     * 
     * @param string $path The API path to append to the endpoint URL.
     * @param string $method The HTTP method to use (POST, PUT, DELETE). Default is 'POST'.
     * @param array $data The data to send in the body of the request.
     * 
     * @return array|string The response from the API, or an error message on failure.
     */
    public static function send_request($path, $method = 'POST', $data = [], $type = 1)
    {
        // Construct the complete URL
        $url = self::get_endpoint() . $path; // Ensure no leading slash in $path
        $settings = get_option('philia_api_settings');

        if( empty( $settings['api_key'] ) ) {
            Philia_API_Log::error("Error occurred: The API key is not set. Please configure the API key in the Philia API settings.");
            return;
        }

        $args = array(
            'method'    => $method,
            'headers'   => array(
                'Content-Type' => 'application/json',
            )
        );

        if( !empty( $data ) )
            $args['body'] = json_encode( $data );

        if( $type == 2 )
            $args['headers']['X-API-Key'] = esc_attr( $settings['api_key'] );
        else
            $args['Authorization'] = 'Api-Key ' . esc_attr( $settings['api_key'] );

        // Log the request being sent
        //Philia_API_Log::info("Sending request to URL: " . $url . " with data: " . json_encode( $data ) );

        // Send POST request using WordPress's wp_remote_post function
        $response = wp_remote_request($url, $args);

        // Log successful response
        //Philia_API_Log::info("Received API Response From $path: " . print_r( $response, true ) );

        // Return the API response as an associative array
        return $response;
    }
}

if (class_exists('Philia_API\Philia_Core_API')) {
    Philia_Core_API::instance();
}