<?php
/**
 * Philia Product API
 * Handles interactions with the 3rd-party API for user registration, updates, and other user actions.
 * 
 * Package: Philia
 * Author: Philia
 * Last Modified Time: 2024/12/09 12:03:28
 * License: MIT
 */

namespace Philia_API;

class Philia_Product_API extends Philia_Core_API {

    /**
     * Static instance of the class for the singleton pattern.
     * 
     * @var Philia_Product_API|null
     */
    protected static $_instance = null;

    /**
     * Singleton instance
     * 
     * Ensures only one instance of the class is created.
     * 
     * @return Philia_Product_API The instance of this class.
     */
    public static function instance() {
        if (is_null(self::$_instance)) {
            self::$_instance = new self();
        }
        return self::$_instance;
    }
    
    /**
     * Constructor.
     * Initializes the class by retrieving the API URL from WordPress settings.
     * 
     */
    public function __construct()
    {
        // Product-related hooks
        add_action('save_post_product', array(&$this, 'on_product_save'), 10, 1);
        add_action('before_delete_post', array(&$this, 'on_product_delete'), 10, 1);
    }

    /**
     * Prepare product data for API requests.
     * 
     * @param int $product_id The ID of the WooCommerce product.
     * 
     * @return array The prepared product data for the API.
     */
    public static function prepare_product_data($product_id)
    {
        $product = wc_get_product( $product_id );

        // prepare attributes
        $attributes = array();
		$product_attributes = self::get_attributes( $product );
		if( !empty( $product_attributes ) )
		{
			foreach( $product_attributes as $attribute )
			{
				$attributes[] = array(
					'name' => $attribute['name'],
					'values' => array_column( $attribute['options'], 'name')
				);
			}
		}

        // prepare image url
		$imageUrl = wp_get_attachment_image_src( get_post_thumbnail_id( $product->get_id() ) , 'single-post-thumbnail' );

        // prepare image gallery urls
		$image_gallery_urls = [];
		$galleryImageIds = $product->get_gallery_image_ids();
		foreach ( $galleryImageIds as $imageId ) {
			$image_gallery_urls[] = wp_get_attachment_url( $imageId );
		}

        // prepare categories
		$categories = wp_get_post_terms( $product->get_id(), 'product_cat' );
		$category_list = [];
		foreach ( $categories as $category ) {
			$category_list[] = $category->name;
		}

        // prepare tags
		$tags = wp_get_post_terms( $product->get_id(), 'product_tag' );
		$tag_list = [];
		foreach ( $tags as $tag ) {
			$tag_list[] = $tag->name;
		}

        $api_data = array(
            'productId' =>  $product->get_id(),
            'title' =>  $product->get_title(),
            'description'  =>  $product->get_description(),
            'shortDescription'     =>  $product->get_short_description(),
            'type'     =>  $product->get_type(),
            'sku' => $product->get_sku(),
            'stockQuantity' => $product->get_stock_quantity(),
            'upSells' => $product->get_upsell_ids(),
            'attributes' => $attributes,
            'imageUrl' => $imageUrl ? $imageUrl[0] : NULL,
            'imageGalleryUrls' => $image_gallery_urls,
            'categories' => $category_list,
            'tags' => $tag_list
        );

        if( $product->get_type() == 'simple' )
        {
            $api_data['isVirtual'] = $product->is_virtual();
            $api_data['isDownloadable'] = $product->is_downloadable();
    
            $api_data['regularPrice'] = floatval( preg_replace( '#[^\d.]#', '', $product->get_regular_price() ) );
            $api_data['salePrice'] = floatval( preg_replace( '#[^\d.]#', '', $product->get_sale_price() ) );
    
            $api_data['salePriceDatesFrom'] = '';
            if ( $product->get_date_on_sale_from() ) {
                $api_data['salePriceDatesFrom'] = $product->get_date_on_sale_from()->date("Y-m-d H:i:s");
            }
    
            $api_data['salePriceDatesTo'] = '';
            if ( $product->get_date_on_sale_to() ) {
                $api_data['salePriceDatesTo'] = $product->get_date_on_sale_to()->date("Y-m-d H:i:s");
            }
    
            $api_data['taxStatus'] = $product->get_tax_status();
            $api_data['taxClass']  = $product->get_tax_class();
    
            $api_data['isSoldIndividually'] = $product->is_sold_individually();
    
            $api_data['weight'] = floatval( $product->get_weight() );
            $api_data['length'] = floatval( $product->get_length() );
            $api_data['width']  = floatval( $product->get_width() );
            $api_data['height'] = floatval( $product->get_height() );
    
            $api_data['shippingClass'] = $product->get_shipping_class();
            $api_data['crossSells'] = $product->get_cross_sell_ids();
        }
        elseif( $product->get_type() == 'variable' )
        {
            $api_data['taxStatus'] = $product->get_tax_status();
            $api_data['taxClass']  = $product->get_tax_class();
    
            $api_data['isSoldIndividually'] = $product->is_sold_individually();
    
            $api_data['weight'] = floatval( $product->get_weight() );
            $api_data['length'] = floatval( $product->get_length() );
            $api_data['width']  = floatval( $product->get_width() );
            $api_data['height'] = floatval( $product->get_height() );
    
            $api_data['shippingClass'] = $product->get_shipping_class();
    
            $api_data['crossSells'] = $product->get_cross_sell_ids();
    
            $api_data['variations'] = [];
            $variation_data = [];
            foreach ( $product->get_children() as $variationId ) {
                $productVariation = wc_get_product( $variationId );

                $variation_data['variationId'] = $variationId;

                $attributes = [];
                $product_attributes = self::get_attributes( $productVariation );
                if( !empty( $product_attributes ) )
                {
                    foreach( $product_attributes as $attribute )
                    {
                        // Optionally, remove the 'pa_' prefix from the attribute label
                        $attributes[] = array(
                            'name' => $attribute['name'],
                            'values' => $attribute['option']
                        );
                    }
                }
                $variation_data['attributes'] = $attributes;

                $imageUrl = $imageUrl = wp_get_attachment_image_src( get_post_thumbnail_id( $productVariation->get_id() ) , 'single-post-thumbnail' );
                $variation_data['imageUrl'] = $imageUrl ? $imageUrl[0] : NULL;

                $variation_data['sku'] = $productVariation->get_sku();

                $variation_data['isDownloadable'] = $productVariation->is_downloadable();
                $variation_data['isVirtual']      = $productVariation->is_virtual();

                $variation_data['regularPrice'] = floatval( preg_replace( '#[^\d.]#', '', $productVariation->get_regular_price() ) );
                $variation_data['salePrice']    = floatval( preg_replace( '#[^\d.]#', '', $productVariation->get_sale_price() ) );

                $variation_data['salePriceDatesFrom'] = '';
                if ( $productVariation->get_date_on_sale_from() ) {
                    $variation_data['salePriceDatesFrom'] = $productVariation->get_date_on_sale_from()->date('Y-m-d H:i:s');
                }

                $variation_data['salePriceDatesTo'] = '';
                if ( $productVariation->get_date_on_sale_to() ) {
                    $variation_data['salePriceDatesTo'] = $productVariation->get_date_on_sale_to()->date('Y-m-d H:i:s');
                }

                $variation_data['stockQuantity'] = $productVariation->get_stock_quantity();

                $variation_data['weight'] = floatval( $productVariation->get_weight() );
                $variation_data['length'] = floatval( $productVariation->get_length() );
                $variation_data['width']  = floatval( $productVariation->get_width() );
                $variation_data['height'] = floatval( $productVariation->get_height() );

                $variation_data['shippingClass'] = $productVariation->get_shipping_class();

                $variation_data['description'] = $productVariation->get_description();
                $api_data['variations'][] = $variation_data;
            }
        }
        elseif( $product->get_type() == 'grouped' )
        {
            $api_data['groupedProducts'] = $product->get_children();

            $api_data['regularPrice'] = 0;
            $api_data['salePrice'] = 0;
            foreach ( $api_data['groupedProducts'] as $product ) {
                $api_data['regularPrice'] += floatval( preg_replace( '#[^\d.]#', '', $product->get_regular_price() ) );
                $api_data['salePrice'] += floatval( preg_replace( '#[^\d.]#', '', $product->get_sale_price() ) );
            }
        }

        return $api_data;
    }

    /**
     * Handles product creation or update actions when a product is saved.
     *
     * @param int $product_id The ID of the WooCommerce product being saved.
     * 
     * @return void
     */
    public function on_product_save( $product_id )
    {
        // Verify it's not an autosave or quick edit
        if (defined('DOING_AUTOSAVE') && DOING_AUTOSAVE) {
            return;
        }

        // Check if this is a product update or a new creation
        if (get_post_status($product_id) == 'publish') {
            if ($this->is_existing_product($product_id)) {
                $this->on_product_update($product_id);
            } else {
                $this->on_product_create($product_id);
            }
        }
    }

    private function is_existing_product( $product_id ) {
        $all_product_ids = get_option('philia_products_ids', []);
        return (!empty( $all_product_ids ) && in_array($product_id, $all_product_ids));
    }

    /**
     * Handle product creation.
     * 
     * @param int $product_id The ID of the created product.
     */
    public function on_product_create($product_id)
    {
        $api_data = self::prepare_product_data($product_id);

        // Send API request to create product
        $response = parent::send_request('wordpress/woocommerce/product/create/', 'POST', [ $api_data ] );

        if (is_wp_error($response) ) {
            Philia_API_Log::error("Error in 'product/create/' for Product " . $product_id);
        } else {
            Philia_API_Log::info("Product created successfully via API - Product ID: " . $product_id);
            self::add_product_id($product_id);
        }
    }

    /**
     * Handle product update.
     * 
     * @param int $product_id The ID of the updated product.
     */
    public function on_product_update($product_id)
    {
        $api_data = self::prepare_product_data($product_id);

        // Send API request to update product
        $response = parent::send_request('wordpress/woocommerce/product/update/', 'PUT', [ $api_data ] );

        if (is_wp_error($response) ) {
            Philia_API_Log::error("Error in 'product/update/' for Product " . $product_id);
        } else {
            Philia_API_Log::info("Product updated successfully via API - Product ID: " . $product_id);
        }
    }

    /**
     * Handle product deletion.
     * 
     * @param int $post_id The ID of the deleted post (product).
     */
    public function on_product_delete($post_id)
    {
        if (get_post_type($post_id) !== 'product') {
            return; // Skip if the post is not a product
        }

        // Send API request to delete product
        $response = parent::send_request('wordpress/woocommerce/product/delete/' . $post_id, 'DELETE');

        if ( is_wp_error( $response ) ) {
            Philia_API_Log::error("Error in 'product/delete/' for Product " . $post_id);
        } else {
            Philia_API_Log::info("Product deleted successfully via API - Product ID: " . $post_id);
            self::remove_product_id($post_id);
        }
    }

    /**
     * Adds a product ID to the 'philia_products_ids' option list.
     * 
     * @param int $product_id The product ID to be added to the list.
     * @return bool True if the product ID was added, false if it already exists.
     */
    public static function add_product_id($product_id) {
        if (!is_int($product_id)) {
            return false; // Invalid product ID
        }

        return parent::add_item('philia_products_ids', $product_id);
    }

    /**
     * Removes a product ID from the 'philia_products_ids' option list.
     * 
     * @param int $product_id The product ID to be removed from the list.
     * @return bool True if the product ID was removed, false if not found.
     */
    public static function remove_product_id($product_id) {
        if (!is_int($product_id)) {
            return false; // Invalid product ID
        }

        return parent::remove_item('philia_products_ids', $product_id);
    }

    public static function philia_get_attribute_taxonomy_label( $name ) {
		$tax    = get_taxonomy( $name );
		$labels = get_taxonomy_labels( $tax );

		return $labels->singular_name;
	}

    /**
     * Get product attributes.
     *
     * @param object $product WooCommerce product object.
     * @return array List of attributes for the product, including ID, name, and options.
     */
    public static function get_attributes($product) {

        $attributes = array();

		if ( $product->is_type( 'variation' ) ) {
			// Variation attributes.
			foreach ( $product->get_variation_attributes() as $attribute_name => $attribute ) {
				$name = urldecode( str_replace( 'attribute_', '', $attribute_name ) );

				if ( ! $attribute ) {
					continue;
				}

				// Taxonomy-based attributes are prefixed with `pa_`, otherwise simply `attribute_`.
				if ( 0 === strpos( $attribute_name, 'attribute_pa_' ) ) {
					$option_term = get_term_by( 'slug', $attribute, $name );
					$attributes[] = array(
						'id'     => wc_attribute_taxonomy_id_by_name( $name ),
						'name'   => self::philia_get_attribute_taxonomy_label( $name ),
						'option' => $option_term && ! is_wp_error( $option_term ) ? $option_term->name : $attribute,
					);
				} else {
					$attributes[] = array(
						'id'     => 0,
						'name'   => urldecode( $name ),
						'option' => $attribute,
					);
				}
			}
		} else {
			foreach ( $product->get_attributes() as $attribute ) {
				if ( $attribute['is_taxonomy'] ) {
					$attributes[] = array(
						'id'        => wc_attribute_taxonomy_id_by_name( $attribute['name'] ),
						'name'      => self::philia_get_attribute_taxonomy_label( $attribute['name'] ),
						'position'  => (int) $attribute['position'],
						'visible'   => (bool) $attribute['is_visible'],
						'variation' => (bool) $attribute['is_variation'],
						'options'   => self::philia_get_attribute_options( $product->get_id(), $attribute ),
					);
				} else {
					$attributes[] = array(
						'id'        => 0,
						'name'      => urldecode( $attribute['name'] ),
						'position'  => (int) $attribute['position'],
						'visible'   => (bool) $attribute['is_visible'],
						'variation' => (bool) $attribute['is_variation'],
						'options'   => self::philia_get_attribute_options( $product->get_id(), $attribute ),
					);
				}
			}
		}

		return $attributes;
	}

    /**
     * Get attribute options for a given product attribute.
     *
     * @param int $product_id The ID of the WooCommerce product.
     * @param array $attribute The attribute data containing its metadata.
     * @return array An array of attribute options or an empty array if none are available.
     */
    public static function philia_get_attribute_options( $product_id, $attribute )
    {
		if ( isset( $attribute['is_taxonomy'] ) && $attribute['is_taxonomy'] )
        {
			$options_list = [];
			$product_terms = wc_get_product_terms( $product_id, $attribute['name'] );
			if(!empty($product_terms))
			{
				foreach($product_terms as $index => $term)
				{
					$product_term_meta_list = get_term_meta($term->term_id);
					if(!empty($product_term_meta_list))
					{
						$term_list = [];
						foreach($product_term_meta_list as $key => $term_meta_element)
						{
							if($key=="case_content")
								$term_data[$key] = unserialize( $term_meta_element[0] );
							else
								$term_data[$key] = $term_meta_element[0];
						}
					}
					$options[] = (object) array_merge( (array)$term, $term_data );
				}
			}
			//return 
			return $options;
		} elseif ( isset( $attribute['value'] ) ) {
			return array_map( 'trim', explode( '|', $attribute['value'] ) );
		}

		return array();
	}
}

if (class_exists('Philia_API\Philia_Product_API')) {
    Philia_Product_API::instance();
}