<?php
/**
 * Philia Product Category API
 * Handles interactions with the 3rd-party API for product category creation, updates, and deletions.
 * 
 * Package: Philia
 * Author: Philia
 * Last Modified Time: 2024/12/09 12:03:52
 * License: MIT
 */
 
namespace Philia_API;

class Philia_Product_Category_API extends Philia_Core_API {

    /**
     * Static instance of the class for the singleton pattern.
    * 
    * @var Philia_Product_Category_API|null
    */
    protected static $_instance = null;

    /**
     * Singleton instance
    * 
    * Ensures only one instance of the class is created.
    * 
    * @return Philia_Product_Category_API The instance of this class.
    */
    public static function instance() {
        if (is_null(self::$_instance)) {
            self::$_instance = new self();
        }
        return self::$_instance;
    }
    
    /**
     * Constructor.
    * Initializes the class by registering WooCommerce product category-related hooks.
    * 
    */
    public function __construct()
    {
        // Add product category-related hooks
        add_action('create_product_cat', array(&$this, 'on_category_create'), 10, 1);
        add_action('edited_product_cat', array(&$this, 'on_category_update'), 10, 1);
        add_action('delete_product_cat', array(&$this, 'on_category_delete'), 10, 1);
    }

    /**
     * Prepare product category data for API requests.
    * 
    * @param int $category_id The product category ID.
    * 
    * @return array The prepared category data for the API.
    */
    public static function prepare_category_data($category_id)
    {
        $category = get_term($category_id, 'product_cat');
        
        if (is_wp_error($category)) {
            return [];
        }

        return array(
            'categoryId' => $category->term_id,
            'name' => $category->name,
            'parentId' => $category->parent
        );
    }

    /**
     * Handle product category creation.
    * 
    * @param int $category_id The ID of the created category.
    */
    public function on_category_create($category_id)
    {
        $api_data = self::prepare_category_data($category_id);

        // Call the 3rd-party API to create the category
        $response = parent::send_request('wordpress/woocommerce/category/create/', 'POST', [ $api_data ]);

        if (is_wp_error($response) || wp_remote_retrieve_response_code($response) != 201) {
            Philia_API_Log::error("Response Error From 'category/create/' - Category " . $category_id . " : " . print_r(wp_remote_retrieve_body($response), true));
        } else {
            Philia_API_Log::info("Response Success From 'category/create/' - Category " . $category_id);
            // Add category ID to the options
            self::add_category_id($category_id);
        }
    }

    /**
     * Handle product category updates.
    * 
    * @param int $category_id The ID of the updated category.
    */
    public function on_category_update($category_id)
    {
        $api_data = self::prepare_category_data($category_id);

        // Call the 3rd-party API to update the category
        $response = parent::send_request('wordpress/woocommerce/category/update/', 'PUT', [$api_data]);

        if (is_wp_error($response) || wp_remote_retrieve_response_code($response) != 201) {
            Philia_API_Log::error("Response Error From 'category/update/' - Category " . $category_id . " : " . print_r(wp_remote_retrieve_body($response), true));
        } else {
            Philia_API_Log::info("Response Success From 'category/update/' - Category " . $category_id);
        }
    }

    /**
     * Handle product category deletion.
    * 
    * @param int $category_id The ID of the deleted category.
    */
    public function on_category_delete($category_id)
    {
        // Call the 3rd-party API to delete the category
        $response = parent::send_request('wordpress/woocommerce/category/delete/' . $category_id . '/', 'DELETE');

        if (is_wp_error($response)) {
            Philia_API_Log::error("Response Error From 'category/delete/' - Category " . $category_id . " : " . print_r(wp_remote_retrieve_body($response), true));
        } else {
            Philia_API_Log::info("Response Success From 'category/delete/' - Category " . $category_id);
            // Remove category ID from the options
            self::remove_category_id($category_id);
        }
    }

    /**
     * Adds a product category ID to the 'philia_category_ids' option list if it doesn't already exist.
    * 
    * @param int $category_id The category ID to be added to the list.
    * @return bool True if the category ID was added, false if it already exists.
    */
    public static function add_category_id($category_id) {
        if (!is_int($category_id)) {
            return false; // Invalid category ID
        }

        // Use the parent function to add the category ID to 'philia_category_ids'
        return parent::add_item('philia_products_categories_ids', $category_id);
    }

    /**
     * Removes a product category ID from the 'philia_category_ids' option list if it exists.
    * 
    * @param int $category_id The category ID to be removed from the list.
    * @return bool True if the category ID was removed, false if not found.
    */
    public static function remove_category_id($category_id) {
        if (!is_int($category_id)) {
            return false; // Invalid category ID
        }

        // Use the parent function to remove the category ID from 'philia_category_ids'
        return parent::remove_item('philia_products_categories_ids', $category_id);
    }
}

if (class_exists('Philia_API\Philia_Product_Category_API')) {
    Philia_Product_Category_API::instance();
}
