<?php
/**
 * Philia User API
 * Handles interactions with the 3rd-party API for user registration, updates, and other user actions.
 * 
 * Package: Philia
 * Author: Philia
 * Last Modified Time: 2024/12/09 12:04:31
 * License: MIT
 */

namespace Philia_API;

class Philia_User_API extends Philia_Core_API {

    /**
     * Static instance of the class for the singleton pattern.
     * 
     * @var Philia_User_API|null
     */
    protected static $_instance = null;

    /**
     * Singleton instance
     * 
     * Ensures only one instance of the class is created.
     * 
     * @return Philia_User_API The instance of this class.
     */
    public static function instance() {
        if (is_null(self::$_instance)) {
            self::$_instance = new self();
        }
        return self::$_instance;
    }
    
    /**
     * Constructor.
     * Initializes the class by retrieving the API URL from WordPress settings.
     * 
     */
    public function __construct()
    {
        // Add user-related hooks
        add_action('user_register', array(&$this, 'on_user_register'), 10, 1);
        add_action('profile_update', array(&$this, 'on_profile_update'), 10, 2);
        add_action('delete_user', array(&$this, 'on_user_delete'), 10, 1);
        //add_action('wp_login', array(&$this, 'on_user_login'), 10, 2);
    }

    /**
     * Prepare user data for API requests.
     * 
     * @param WP_User|int $user The user object or user ID.
     * 
     * @return array The prepared user data for the API.
     */
    public static function prepare_user_data( $user )
    {
        if ( is_int( $user ) ) {
            $user = get_userdata( $user );  // If only the user ID is passed, retrieve the user object
        }

		if( !empty( $user->first_name ) && !empty( $user->last_name ) )
		{
			$first_name = $user->first_name;
			$last_name = $user->last_name;
		}
		elseif( empty( $user->first_name ) && !empty( $user->last_name ) )
		{
			$first_name = 'user';
			$last_name = $user->last_name;
		}
		elseif( !empty( $user->first_name ) && empty( $user->last_name ) )
		{
			$first_name = $user->first_name;
			$last_name = strval( $user->ID );
		}
		else
		{
			$first_name = 'user';
			$last_name = strval( $user->ID );
		}
		
		if( !empty( $user->user_email ) )		
			$user_email = $user->user_email;
		else
            $user_email = self::generate_random_email_address();

        $settings = get_option('philia_api_settings');
		if( !empty( $settings['phone_meta'] ) )
		{
			$user_phone = get_user_meta( $user->ID, $settings['phone_meta'] , true );
			if( !empty( $user_phone ) )
			{
				$user_mobile = $user_phone;
			}
			else
			{
				$user_mobile = "09123456789";
			}
		}
		else
		{
			$user_mobile = "09123456789";
		}

        $api_data = array(
            'userId'    =>  $user->ID,
            'firstName' =>  $first_name,
            'lastName'  =>  $last_name,
            'email'     =>  $user_email,
            'phone'     =>  $user_mobile
        );

        return $api_data;
    }

    /**
     * Handle user registration.
     * 
     * @param int $user_id The ID of the registered user.
     */
    public function on_user_register($user_id)
    {
        $api_data = array();

        $api_data[] = self::prepare_user_data( $user_id );

        // Call the 3rd-party API to create the user
        $response = parent::send_request('wordpress/woocommerce/user/create/', 'POST', $api_data );

        if ( is_wp_error( $response ) && wp_remote_retrieve_response_code( $response ) != 201 ) {
            Philia_API_Log::error("Response Error From 'user/create/' - User " . $user_id . " : " . print_r( wp_remote_retrieve_body( $response ), true ) );
        } else {
            Philia_API_Log::info("Response Success From 'user/create/' - User " . $user_id );

            // add to option
            self::add_user_id( $user_id );
        }
    }

    /**
     * Handle user profile updates.
     * 
     * @param int $user_id The ID of the updated user.
     * @param WP_User $old_user_data The old user data before the update.
     */
    public function on_profile_update($user_id, $old_user_data)
    {
        $api_data = self::prepare_user_data( $user_id );

        // Call the 3rd-party API to update the user
        $response = parent::send_request('wordpress/woocommerce/user/update/', 'PUT', [ $api_data ] );

        if ( is_wp_error( $response ) && wp_remote_retrieve_response_code( $response ) != 201 ) {
            Philia_API_Log::error("Response Error From 'user/update/' - User " . $user_id . " : " . print_r( wp_remote_retrieve_body( $response ), true ) );
        } else {
            Philia_API_Log::info("Response Success From 'user/update/' - User " . $user_id );
        }
    }

    /**
     * Handle user deletion.
     * 
     * @param int $user_id The ID of the deleted user.
     */
    public function on_user_delete($user_id)
    {
        // Call the 3rd-party API to delete the user
        $response = parent::send_request('wordpress/woocommerce/user/delete/' . $user_id . '/' , 'DELETE');

        if ( is_wp_error( $response )) {
            Philia_API_Log::error("Response Error From 'user/delete/' - User " . $user_id . " : " . print_r( wp_remote_retrieve_body( $response ), true ) );
        } else {
            Philia_API_Log::info("Response Success From 'user/delete/' - User " . $user_id );
            
            // Remove from option
            self::remove_user_id( $user_id );
        }
    }

    /**
     * Handle user login.
     * 
     * @param string $user_login The username of the user.
     * @param WP_User $user The WP_User object of the logged-in user.
     */
    public function on_user_login($user_login, $user)
    {
        $api_data = self::prepare_user_data( $user );

        // Call the 3rd-party API to log the user login
        $response = parent::send_request('wordpress/woocommerce/user/login/', 'POST', [ $api_data ] );

        if ( is_wp_error( $response ) && wp_remote_retrieve_response_code( $response ) != 200 ) {
            Philia_API_Log::error("Response Error From 'user/login/' - User " . $user_id . " : " . print_r( wp_remote_retrieve_body( $response ), true ) );
        } else {
            Philia_API_Log::info("Response Success From 'user/login/' - User " . $user_id );
        }
    }

    /**
     * Adds a user ID to the 'philia_users_ids' option list if it doesn't already exist.
     * 
     * @param int $user_id The user ID to be added to the list.
     * @return bool True if the user ID was added, false if it already exists.
     */
    public static function add_user_id($user_id) {
        if (!is_int($user_id)) {
            return false; // Invalid user ID
        }
        
        // Use the parent function to add the user ID to 'philia_users_ids'
        return parent::add_item('philia_users_ids', $user_id);
    }

    /**
     * Removes a user ID from the 'philia_users_ids' option list if it exists.
     * 
     * @param int $user_id The user ID to be removed from the list.
     * @return bool True if the user ID was removed, false if not found.
     */
    public static function remove_user_id($user_id) {
        if (!is_int($user_id)) {
            return false; // Invalid user ID
        }
        
        // Use the parent function to remove the user ID from 'philia_users_ids'
        return parent::remove_item('philia_users_ids', $user_id);
    }

    /**
     * Generate a random email address based on the site's domain.
     * 
     * @return string The generated random email address.
     */
    public static function generate_random_email_address()
    {
        // Get the site URL without 'http://' or 'https://'
        $site_url = get_site_url();
        $domain = parse_url($site_url, PHP_URL_HOST);
        
        // Generate a random username (you can customize the length as needed)
        $length = 8; // Length of the random username
        $characters = 'abcdefghijklmnopqrstuvwxyz0123456789'; // Characters to choose from
        $username = '';
        
        // Generate the random username
        for ($i = 0; $i < $length; $i++) {
            $username .= $characters[mt_rand(0, strlen($characters) - 1)];
        }
        
        // Create the email address
        $email_address = sanitize_user($username) . '@' . $domain;
        
        return $email_address;
    }
}

if (class_exists('Philia_API\Philia_User_API')) {
    Philia_User_API::instance();
}